"""Import all constants here for easy access"""

from .compression import (
    FASTQ_DELTA,
    FASTQ_FIRST_READ_SUFFIX,
    FASTQ_SECOND_READ_SUFFIX,
    SPRING_SUFFIX,
)
from .constants import (
    CAPTUREKIT_CANCER_OPTIONS,
    CAPTUREKIT_OPTIONS,
    CASE_ACTIONS,
    COLLABORATORS,
    COMBOS,
    CONTAINER_OPTIONS,
    DEFAULT_CAPTURE_KIT,
    FLOWCELL_STATUS,
    PREP_CATEGORIES,
    SEX_OPTIONS,
    STATUS_OPTIONS,
    DataDelivery,
    Pipeline,
)
from .gene_panel import MASTER_LIST
from .paths import TMP_DIR
from .priority import PRIORITY_MAP, PRIORITY_OPTIONS, REV_PRIORITY_MAP
from .process import EXIT_FAIL, EXIT_SUCCESS, RETURN_SUCCESS
from .sample_sources import ANALYSIS_SOURCES, METAGENOME_SOURCES
from .symbols import NO_PARENT, SINGLE_QUOTE, SPACE
from .tags import HK_FASTQ_TAGS
